@@ -58,12 +58,14 @@ DBusExtendedAbstractInterface::~DBusExtendedAbstractInterface()
 {
 }
 
-void DBusExtendedAbstractInterface::setSync(bool sync)
+void DBusExtendedAbstractInterface::setSync(bool sync) { setSync(sync, true); }
+
+void DBusExtendedAbstractInterface::setSync(bool sync, bool autoStart)
 {
     m_sync = sync;
 
     // init all properties
-    if (!m_sync && !isValid())
+    if (autoStart && !m_sync && !isValid())
         startServiceProcess();
 }
 
@@ -168,7 +170,11 @@ QVariant DBusExtendedAbstractInterface::internalPropGet(const char *propname, vo
     }
 
     if (m_sync) {
-        return property(propname);
+        QVariant ret = property(propname);
+
+        QMetaType::construct(ret.userType(), propertyPtr, ret.constData());
+
+        return ret;
     } else {
         if (!isValid()) {
             QString errorMessage = QStringLiteral("This Extended DBus interface is not valid yet.");
@@ -286,13 +292,16 @@ void DBusExtendedAbstractInterface::asyncSetProperty(const QString &propertyName
 
 void DBusExtendedAbstractInterface::startServiceProcess()
 {
-    Q_ASSERT(!isValid());
+    const QString &servName = service();
 
-//    if (!m_useCache)
-//        qWarning() << QStringLiteral("Maybe your dbus service is not permanent process, you need to open cache with setUseCache(true)");
+    if (isValid())
+    {
+        qWarning() << "Service" << servName << "is already started.";
+        return;
+    }
 
     QDBusMessage msg = QDBusMessage::createMethodCall("org.freedesktop.DBus", "/", *dBusInterface(), QStringLiteral("StartServiceByName"));
-    msg << service() << quint32(0);
+    msg << servName << quint32(0);
     QDBusPendingReply<quint32> async = connection().asyncCall(msg);
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(async, this);
 
@@ -310,7 +319,7 @@ void DBusExtendedAbstractInterface::onStartServiceProcessFinished(QDBusPendingCa
 
     QDBusPendingReply<quint32> reply = *w;
 
-    emit serviceStartFinished(reply.value());
+    Q_EMIT serviceStartFinished(reply.value());
 
     w->deleteLater();
 }
@@ -332,13 +341,13 @@ void DBusExtendedAbstractInterface::onAsyncPropertyFinished(QDBusPendingCallWatc
                                     &m_lastExtendedError);
 
         if (m_lastExtendedError.isValid()) {
-            emit propertyInvalidated(watcher->asyncProperty());
+            Q_EMIT propertyInvalidated(watcher->asyncProperty());
         } else {
-            emit propertyChanged(watcher->asyncProperty(), value);
+            Q_EMIT propertyChanged(watcher->asyncProperty(), value);
         }
     }
 
-    emit asyncPropertyFinished(watcher->asyncProperty());
+    Q_EMIT asyncPropertyFinished(watcher->asyncProperty());
     watcher->deleteLater();
 }
 
@@ -355,13 +364,13 @@ void DBusExtendedAbstractInterface::onAsyncSetPropertyFinished(QDBusPendingCallW
         m_lastExtendedError = QDBusError();
     }
 
-    emit asyncSetPropertyFinished(watcher->asyncProperty());
+    Q_EMIT asyncSetPropertyFinished(watcher->asyncProperty());
 
     // Resetting the property to its previous value after sending the
     // finished signal
     if (reply.isError()) {
         m_lastExtendedError = QDBusError();
-        emit propertyChanged(watcher->asyncProperty(), watcher->previousValue());
+        Q_EMIT propertyChanged(watcher->asyncProperty(), watcher->previousValue());
     }
 
     watcher->deleteLater();
@@ -379,7 +388,7 @@ void DBusExtendedAbstractInterface::onAsyncGetAllPropertiesFinished(QDBusPending
         m_lastExtendedError = QDBusError();
     }
 
-    emit asyncGetAllPropertiesFinished();
+    Q_EMIT asyncGetAllPropertiesFinished();
 
     if (!reply.isError()) {
         onPropertiesChanged(interface(), reply.value(), QStringList());
@@ -403,9 +412,9 @@ void DBusExtendedAbstractInterface::onPropertiesChanged(const QString& interface
                 QVariant value = demarshall(interface(), metaObject()->property(propertyIndex), i.value(), &m_lastExtendedError);
 
                 if (m_lastExtendedError.isValid()) {
-                    emit propertyInvalidated(i.key());
+                    Q_EMIT propertyInvalidated(i.key());
                 } else {
-                    emit propertyChanged(i.key(), value);
+                    Q_EMIT propertyChanged(i.key(), value);
                 }
             }
 
@@ -418,7 +427,7 @@ void DBusExtendedAbstractInterface::onPropertiesChanged(const QString& interface
                 qDebug() << Q_FUNC_INFO << "Got unknown invalidated property" <<  *j;
             } else {
                 m_lastExtendedError = QDBusError();
-                emit propertyInvalidated(*j);
+                Q_EMIT propertyInvalidated(*j);
             }
 
             ++j;
@@ -431,12 +440,12 @@ void DBusExtendedAbstractInterface::onDBusNameOwnerChanged(const QString &name,
     if (name == service() && oldOwner.isEmpty())
     {
         m_dbusOwner = newOwner;
-        emit serviceValidChanged(true);
+        Q_EMIT serviceValidChanged(true);
     }
     else if (name == m_dbusOwner && newOwner.isEmpty())
     {
         m_dbusOwner.clear();
-        emit serviceValidChanged(false);
+        Q_EMIT serviceValidChanged(false);
     }
 }
 
